@@ -173,6 +173,7 @@ gerbv_destroy_image(gerbv_image_t *image)
     	gerbv_layer_t *tempLayer = layer;
     	
     	layer = layer->next;
+    	g_free (tempLayer->name);
     	g_free (tempLayer);
     }
     for (state = image->states; state != NULL; ) {
@@ -295,7 +296,7 @@ gerbv_image_dump(gerbv_image_t const* image)
     while (net){
 	printf(_("(%f,%f)->(%f,%f) with %d ("), net->start_x, net->start_y,
 	       net->stop_x, net->stop_y, net->aperture);
-	printf(_(gerbv_interpolation_name(net->interpolation)));
+	printf("%s", _(gerbv_interpolation_name(net->interpolation)));
 	gerbv_image_aperture_state(net->aperture_state);
 	printf(")\n");
 	net = net->next;
@@ -312,6 +313,7 @@ gerbv_image_return_new_layer (gerbv_layer_t *previousLayer)
     previousLayer->next = newLayer;
     /* clear this boolean so we only draw the knockout once */
     newLayer->knockout.firstInstance = FALSE;
+    newLayer->name = NULL;
     newLayer->next = NULL;
     
     return newLayer;
@@ -402,7 +404,6 @@ gerbv_image_copy_all_nets (gerbv_image_t *sourceImage,
 		err_unknown_macro_aperture = 0,
 		err_rotate_oval = 0,
 		err_rotate_rect = 0;
-	guint i;
 
 	if (trans && (trans->mirrorAroundX || trans->mirrorAroundY)) {
 		if (sourceImage->layertype != GERBV_LAYERTYPE_DRILL) {
@@ -429,7 +430,7 @@ gerbv_image_copy_all_nets (gerbv_image_t *sourceImage,
 
 		trans_apers = g_new (int, aper_last_id + 1);
 		/* Initialize trans_apers array */
-		for (i = 0; i < aper_last_id + 1; i++)
+		for (int i = 0; i < aper_last_id + 1; i++)
 			trans_apers[i] = -1;
 	}
 
@@ -476,7 +477,7 @@ gerbv_image_copy_all_nets (gerbv_image_t *sourceImage,
 
 		/* Check if we need to translate the aperture number */
 		if (translationTable) {
-			for (i = 0; i < translationTable->len; i++) {
+			for (guint i = 0; i < translationTable->len; i++) {
 				gerb_translation_entry_t translationEntry;
 
 				translationEntry =
@@ -704,7 +705,7 @@ gerbv_image_copy_all_nets (gerbv_image_t *sourceImage,
 					break;
 
 				case GERBV_APTYPE_MACRO_OUTLINE:
-					for (i = 0; i < 1 + sam->parameter[
+					for (int i = 0; i < 1 + sam->parameter[
 							OUTLINE_NUMBER_OF_POINTS]; i++) {
 						sam->parameter[OUTLINE_X_IDX_OF_POINT(i)] *=
 								trans->scaleX;
@@ -1215,10 +1216,9 @@ gerbv_image_create_window_pane_objects (gerbv_image_t *image, gdouble lowerLeftX
 gboolean
 gerbv_image_reduce_area_of_selected_objects (GArray *selectionArray,
 		gdouble areaReduction, gint paneRows, gint paneColumns, gdouble paneSeparation){
-	int i;
 	gdouble minX,minY,maxX,maxY;
 	
-	for (i=0; i<selectionArray->len; i++) {
+	for (guint i=0; i<selectionArray->len; i++) {
 		gerbv_selection_item_t sItem = g_array_index (selectionArray,gerbv_selection_item_t, i);
 		gerbv_image_t *image = sItem.image;
 		gerbv_net_t *currentNet = sItem.net;
@@ -1311,10 +1311,9 @@ gerbv_image_reduce_area_of_selected_objects (GArray *selectionArray,
 
 gboolean
 gerbv_image_move_selected_objects (GArray *selectionArray, gdouble translationX,
-		gdouble translationY) {
-	int i;
-	
-	for (i=0; i<selectionArray->len; i++) {
+		gdouble translationY)
+{
+	for (guint i=0; i<selectionArray->len; i++) {
 		gerbv_selection_item_t sItem = g_array_index (selectionArray,gerbv_selection_item_t, i);
 		gerbv_net_t *currentNet = sItem.net;
 
