@@ -33,9 +33,15 @@
 
 /* ===================  Prototypes ================ */
 gerbv_error_list_t *gerbv_stats_new_error_list(void);
+void gerbv_stats_printf(gerbv_error_list_t *list, gerbv_message_type_t type,
+			int layer, const char *text, ...)
+					__attribute__ ((format (printf, 4, 5)));
 void gerbv_stats_add_error(gerbv_error_list_t *error_list_in,
                            int layer, const char *error_text,
                            gerbv_message_type_t type);
+#define gerbv_escape_char(c) \
+	((char*)(int[]){gerbv_escape_char_return_int((c))})
+int gerbv_escape_char_return_int(char c);
 
 gerbv_aperture_list_t *gerbv_stats_new_aperture_list(void);
 void gerbv_stats_add_aperture(gerbv_aperture_list_t *aperture_list_in,
