@@ -23,90 +23,61 @@
  */
 
 /** \file render.c
-    \brief Rendering support functions for libgerbv
-    \ingroup libgerbv
+    \brief Rendering support functions for gerbv
+    \ingroup gerbv
 */
 
-#ifdef HAVE_CONFIG_H
-#  include <config.h>
-#endif
+#include "gerbv.h"
 
 #ifdef HAVE_STDLIB_H
-#include <stdlib.h>
+# include <stdlib.h>
 #endif
 
 #ifdef HAVE_STRING_H
-#include <string.h>
+# include <string.h>
 #endif
 
 #ifdef HAVE_UNISTD_H
-#include <unistd.h>
+# include <unistd.h>
 #endif
 
 #ifdef HAVE_LIBGEN_H
-#include <libgen.h> /* dirname */
+# include <libgen.h> /* dirname */
 #endif
 
 #include <math.h>
 
 #include "common.h"
-#include "gerbv.h"
 #include "main.h"
 #include "callbacks.h"
 #include "interface.h"
 #include "render.h"
+#include "selection.h"
 
 #ifdef WIN32
-#include <cairo-win32.h>
+# include <cairo-win32.h>
 #elif QUARTZ
-#include <cairo-quartz.h>
+# include <cairo-quartz.h>
 #else
-#include <cairo-xlib.h>
+# include <cairo-xlib.h>
 #endif
-#include <cairo.h>
-#include "draw-gdk.h"
 #include "draw.h"
 
 #define dprintf if(DEBUG) printf
 
-/**Global variable to keep track of what's happening on the screen.
-   Declared extern in gerbv_screen.h
- */
-extern gerbv_screen_t screen;
-
-extern gerbv_render_info_t screenRenderInfo;
-
-/*
-static void
-render_layer_to_cairo_target_without_transforming(cairo_t *cr, gerbv_fileinfo_t *fileInfo, gerbv_render_info_t *renderInfo );
-*/
-
-gboolean
-render_check_scale_factor_limits (void) {
-	if ((screenRenderInfo.scaleFactorX > 20000)||(screenRenderInfo.scaleFactorY > 20000)) {
-		screenRenderInfo.scaleFactorX = 20000;
-		screenRenderInfo.scaleFactorY = 20000;
-		return FALSE;
-	}
-	return TRUE;
-}
+gerbv_render_info_t screenRenderInfo;
 
 /* ------------------------------------------------------ */
 void
-render_zoom_display (gint zoomType, gdouble scaleFactor, gdouble mouseX, gdouble mouseY) {
-	/*double us_midx, us_midy;*/	/* unscaled translation for screen center */
-	/*int half_w, half_h;*/		/* cache for half window dimensions */
+render_zoom_display (gint zoomType, gdouble scaleFactor, gdouble mouseX, gdouble mouseY)
+{
 	gdouble mouseCoordinateX = 0.0;
 	gdouble mouseCoordinateY = 0.0;
 	double oldWidth, oldHeight;
 
-	/*
-	half_w = screenRenderInfo.displayWidth / 2;
-	half_h = screenRenderInfo.displayHeight / 2;
-	*/
-	
 	oldWidth = screenRenderInfo.displayWidth / screenRenderInfo.scaleFactorX;
 	oldHeight = screenRenderInfo.displayHeight / screenRenderInfo.scaleFactorY;
+
 	if (zoomType == ZOOM_IN_CMOUSE || zoomType == ZOOM_OUT_CMOUSE) {
 		/* calculate what user coordinate the mouse is pointing at */
 		mouseCoordinateX = mouseX / screenRenderInfo.scaleFactorX + screenRenderInfo.lowerLeftX;
@@ -114,49 +85,41 @@ render_zoom_display (gint zoomType, gdouble scaleFactor, gdouble mouseX, gdouble
 			screenRenderInfo.scaleFactorY + screenRenderInfo.lowerLeftY;
 	}
 
-	/*
-	us_midx = screenRenderInfo.lowerLeftX + (screenRenderInfo.displayWidth / 2.0 )/
-			screenRenderInfo.scaleFactorX;
-	us_midy = screenRenderInfo.lowerLeftY + (screenRenderInfo.displayHeight / 2.0 )/
-			screenRenderInfo.scaleFactorY;		
-	*/
-	
 	switch(zoomType) {
 		case ZOOM_IN : /* Zoom In */
 		case ZOOM_IN_CMOUSE : /* Zoom In Around Mouse Pointer */
-			screenRenderInfo.scaleFactorX += screenRenderInfo.scaleFactorX/3;
-			screenRenderInfo.scaleFactorY += screenRenderInfo.scaleFactorY/3;
-			(void) render_check_scale_factor_limits ();
+			screenRenderInfo.scaleFactorX = MIN((gdouble)GERBV_SCALE_MAX,
+					(1 + 1/3.0)*screenRenderInfo.scaleFactorX);
+			screenRenderInfo.scaleFactorY = screenRenderInfo.scaleFactorX;
 			screenRenderInfo.lowerLeftX += (oldWidth - (screenRenderInfo.displayWidth /
-				screenRenderInfo.scaleFactorX)) / 2.0;
+						screenRenderInfo.scaleFactorX)) / 2.0;
 			screenRenderInfo.lowerLeftY += (oldHeight - (screenRenderInfo.displayHeight /
-				screenRenderInfo.scaleFactorY)) / 2.0;	
+						screenRenderInfo.scaleFactorY)) / 2.0;
 			break;
 		case ZOOM_OUT :  /* Zoom Out */
 		case ZOOM_OUT_CMOUSE : /* Zoom Out Around Mouse Pointer */
-			if ((screenRenderInfo.scaleFactorX > 10)&&(screenRenderInfo.scaleFactorY > 10)) {
-				screenRenderInfo.scaleFactorX -= screenRenderInfo.scaleFactorX/3;
-				screenRenderInfo.scaleFactorY -= screenRenderInfo.scaleFactorY/3;
-				screenRenderInfo.lowerLeftX += (oldWidth - (screenRenderInfo.displayWidth /
-					screenRenderInfo.scaleFactorX)) / 2.0;
-				screenRenderInfo.lowerLeftY += (oldHeight - (screenRenderInfo.displayHeight /
-					screenRenderInfo.scaleFactorY)) / 2.0;
-			}
+			screenRenderInfo.scaleFactorX = MAX((gdouble)GERBV_SCALE_MIN,
+					(1 - 1/3.0)*screenRenderInfo.scaleFactorX);
+			screenRenderInfo.scaleFactorY = screenRenderInfo.scaleFactorX;
+			screenRenderInfo.lowerLeftX += (oldWidth - (screenRenderInfo.displayWidth /
+						screenRenderInfo.scaleFactorX)) / 2.0;
+			screenRenderInfo.lowerLeftY += (oldHeight - (screenRenderInfo.displayHeight /
+						screenRenderInfo.scaleFactorY)) / 2.0;
 			break;
 		case ZOOM_FIT : /* Zoom Fit */
 			gerbv_render_zoom_to_fit_display (mainProject, &screenRenderInfo);
 			break;
 		case ZOOM_SET : /*explicit scale set by user */
-			screenRenderInfo.scaleFactorX = scaleFactor;
-			screenRenderInfo.scaleFactorY = scaleFactor;
-			(void) render_check_scale_factor_limits ();
+			screenRenderInfo.scaleFactorX =
+				MIN((gdouble)GERBV_SCALE_MAX, scaleFactor);
+			screenRenderInfo.scaleFactorY = screenRenderInfo.scaleFactorX;
 			screenRenderInfo.lowerLeftX += (oldWidth - (screenRenderInfo.displayWidth /
-				screenRenderInfo.scaleFactorX)) / 2.0;
+						screenRenderInfo.scaleFactorX)) / 2.0;
 			screenRenderInfo.lowerLeftY += (oldHeight - (screenRenderInfo.displayHeight /
-				screenRenderInfo.scaleFactorY)) / 2.0;
-		      break;
+						screenRenderInfo.scaleFactorY)) / 2.0;
+			break;
 		default :
-			GERB_MESSAGE(_("Illegal zoom direction %d\n"), zoomType);
+			GERB_MESSAGE(_("Illegal zoom direction %d"), zoomType);
 	}
 
 	if (zoomType == ZOOM_IN_CMOUSE || zoomType == ZOOM_OUT_CMOUSE) {
@@ -183,10 +146,10 @@ render_calculate_zoom_from_outline(GtkWidget *widget, GdkEventButton *event)
 	double centerPointX, centerPointY;
 	int half_x, half_y;		/* cache for half window dimensions */
 
-	x1 = MIN(screen.start_x, event->x);
-	y1 = MIN(screen.start_y, event->y);
-	x2 = MAX(screen.start_x, event->x);
-	y2 = MAX(screen.start_y, event->y);
+	x1 = MIN((gdouble)screen.start_x, event->x);
+	y1 = MIN((gdouble)screen.start_y, event->y);
+	x2 = MAX((gdouble)screen.start_x, event->x);
+	y2 = MAX((gdouble)screen.start_y, event->y);
 	dx = x2-x1;
 	dy = y2-y1;
 
@@ -203,15 +166,17 @@ render_calculate_zoom_from_outline(GtkWidget *widget, GdkEventButton *event)
 		centerPointX = half_x/screenRenderInfo.scaleFactorX + screenRenderInfo.lowerLeftX;
 		centerPointY = (screenRenderInfo.displayHeight - half_y)/screenRenderInfo.scaleFactorY +
 				screenRenderInfo.lowerLeftY;
-		
-		screenRenderInfo.scaleFactorX *= MIN(((double)screenRenderInfo.displayWidth / dx),
-							((double)screenRenderInfo.displayHeight / dy));
+
+		screenRenderInfo.scaleFactorX *=
+			MIN((double)screenRenderInfo.displayWidth / dx,
+				(double)screenRenderInfo.displayHeight / dy);
+		screenRenderInfo.scaleFactorX = MIN((gdouble)GERBV_SCALE_MAX,
+				screenRenderInfo.scaleFactorX);
 		screenRenderInfo.scaleFactorY = screenRenderInfo.scaleFactorX;
-		(void) render_check_scale_factor_limits ();
 		screenRenderInfo.lowerLeftX = centerPointX - (screenRenderInfo.displayWidth /
 					2.0 / screenRenderInfo.scaleFactorX);
 		screenRenderInfo.lowerLeftY = centerPointY - (screenRenderInfo.displayHeight /
-					2.0 / screenRenderInfo.scaleFactorY);				
+					2.0 / screenRenderInfo.scaleFactorY);
 	}
 	render_refresh_rendered_image_on_screen();
 }
@@ -367,16 +332,17 @@ render_toggle_measure_line(void)
 	gdouble start_x, start_y, last_x, last_y;
 	memset(&values, 0, sizeof(values));
 	values.function = GDK_XOR;
+	values.line_width = 6;
 	if (!screen.zoom_outline_color.pixel)
 	 	gdk_colormap_alloc_color(gdk_colormap_get_system(), &screen.zoom_outline_color, FALSE, TRUE);
 	values.foreground = screen.zoom_outline_color;
-	values_mask = GDK_GC_FUNCTION | GDK_GC_FOREGROUND;
+	values_mask = GDK_GC_FUNCTION | GDK_GC_LINE_WIDTH | GDK_GC_FOREGROUND;
 	gc = gdk_gc_new_with_values(screen.drawing_area->window, &values,
 				values_mask);
 	render_board2screen(&start_x, &start_y,
-				screen.measure_start_x, screen.measure_start_y); 
+				screen.measure_start_x, screen.measure_start_y);
 	render_board2screen(&last_x, &last_y,
-				screen.measure_last_x, screen.measure_last_y); 
+				screen.measure_stop_x, screen.measure_stop_y);
 	render_trim_point(&start_x, &start_y, last_x, last_y);
 	render_trim_point(&last_x, &last_y, start_x, start_y);
 	gdk_draw_line(screen.drawing_area->window, gc, start_x,
@@ -389,55 +355,72 @@ render_toggle_measure_line(void)
 void
 render_draw_measure_distance(void)
 {
-	gdouble x1, y1, x2, y2;
 	gdouble dx, dy;
 
-	x1 = MIN(screen.measure_start_x, screen.measure_last_x);
-	y1 = MIN(screen.measure_start_y, screen.measure_last_y);
-	x2 = MAX(screen.measure_start_x, screen.measure_last_x);
-	y2 = MAX(screen.measure_start_y, screen.measure_last_y);
-	dx = (x2 - x1);
-	dy = (y2 - y1);
-    
-	screen.win.lastMeasuredX = dx;
-	screen.win.lastMeasuredY = dy;
+	dx = fabs (screen.measure_start_x - screen.measure_stop_x);
+	dy = fabs (screen.measure_start_y - screen.measure_stop_y);
+
+	screen.measure_last_x = dx;
+	screen.measure_last_y = dy;
 	callbacks_update_statusbar_measured_distance (dx, dy);
 	render_toggle_measure_line();
-} /* draw_measure_distance */
+}
 
 /* ------------------------------------------------------ */
-void render_selection_layer (void){
+static void render_selection (void)
+{
+	gerbv_selection_item_t sel_item;
+	gerbv_fileinfo_t *file;
+	gdouble pixel_width;
 	cairo_t *cr;
-	
-	if (screen.selectionRenderData) 
-		cairo_surface_destroy ((cairo_surface_t *) screen.selectionRenderData);
-	screen.selectionRenderData = 
-		(gpointer) cairo_surface_create_similar ((cairo_surface_t *)screen.windowSurface,
-		CAIRO_CONTENT_COLOR_ALPHA, screenRenderInfo.displayWidth,
-		screenRenderInfo.displayHeight);
-	if (screen.selectionInfo.type != GERBV_SELECTION_EMPTY) {
-		cr= cairo_create(screen.selectionRenderData);
-		gerbv_render_cairo_set_scale_and_translation(cr, &screenRenderInfo);
-		cairo_set_source_rgba (cr, 1.0, 1.0, 1.0, 0.85);
-		/* for now, assume everything in the selection buffer is from one image */
-		gerbv_image_t *matchImage;
-		int j;
-		if (screen.selectionInfo.selectedNodeArray->len > 0) {
-			gerbv_selection_item_t sItem = g_array_index (screen.selectionInfo.selectedNodeArray,
-					gerbv_selection_item_t, 0);
-			matchImage = (gerbv_image_t *) sItem.image;	
-			dprintf("    .... calling render_image_to_cairo_target on selection layer...\n");
-			for(j = mainProject->last_loaded; j >= 0; j--) {
-				if ((mainProject->file[j]) && (mainProject->file[j]->image == matchImage)) {
-					draw_image_to_cairo_target (cr, mainProject->file[j]->image,
-						1.0/MAX(screenRenderInfo.scaleFactorX,
-						screenRenderInfo.scaleFactorY),
-						DRAW_SELECTIONS, &screen.selectionInfo, &screenRenderInfo,
-						TRUE, mainProject->file[j]->transform, TRUE);
-				}
-			}
+	int i;
+	guint j;
+
+	if (selection_length (&screen.selectionInfo) == 0)
+		return;
+
+	if (screen.selectionRenderData)
+		cairo_surface_destroy (
+				(cairo_surface_t *)screen.selectionRenderData);
+
+	screen.selectionRenderData =
+		(gpointer) cairo_surface_create_similar (
+			(cairo_surface_t *)screen.windowSurface,
+			CAIRO_CONTENT_COLOR_ALPHA,
+			screenRenderInfo.displayWidth,
+			screenRenderInfo.displayHeight);
+
+	pixel_width = 1.0/MAX(screenRenderInfo.scaleFactorX,
+				screenRenderInfo.scaleFactorY);
+
+	for (i = mainProject->last_loaded; i >= 0; i--) {
+		file = mainProject->file[i]; 
+
+		if (!file ||
+		(!mainProject->show_invisible_selection && !file->isVisible))
+			continue;
+
+		for (j = 0; j < selection_length (&screen.selectionInfo); j++) {
+			sel_item = selection_get_item_by_index (
+					&screen.selectionInfo, j);
+			if (file->image != sel_item.image)
+				continue;
+
+			/* Have selected image(s) on this file, draw it */
+
+			cr = cairo_create(screen.selectionRenderData);
+			gerbv_render_cairo_set_scale_and_translation(cr,
+					&screenRenderInfo);
+			cairo_set_source_rgba (cr, 1.0, 1.0, 1.0, 0.85);
+			draw_image_to_cairo_target (cr,
+				file->image, pixel_width,
+				DRAW_SELECTIONS, &screen.selectionInfo,
+				&screenRenderInfo, TRUE,
+				file->transform, TRUE);
+			cairo_destroy (cr);
+
+			break;	/* All done, go to next file */
 		}
-		cairo_destroy (cr);
 	}
 }
 
@@ -481,8 +464,6 @@ void render_refresh_rendered_image_on_screen (void) {
 		    cairo_destroy (cr);
 		}
 	    }
-	    /* render the selection layer */
-	    render_selection_layer();
 	    
 	    render_recreate_composite_surface ();
 	}
@@ -493,30 +474,20 @@ void render_refresh_rendered_image_on_screen (void) {
 
 /* ------------------------------------------------------ */
 void
-render_clear_selection_buffer (void){
-	if (screen.selectionInfo.type == GERBV_SELECTION_EMPTY)
-		return;
+render_remove_selected_objects_belonging_to_layer (
+			gerbv_selection_info_t *sel_info, gerbv_image_t *image)
+{
+	guint i;
 
-	g_array_remove_range (screen.selectionInfo.selectedNodeArray, 0,
-		screen.selectionInfo.selectedNodeArray->len);
-	screen.selectionInfo.type = GERBV_SELECTION_EMPTY;
-	callbacks_update_selected_object_message (FALSE);
-}
+	for (i = 0; i < selection_length (sel_info);) {
+		gerbv_selection_item_t sItem =
+			selection_get_item_by_index (sel_info, i);
 
-void
-render_remove_selected_objects_belonging_to_layer (gint index) {
-	int i;
-	
-	for (i=screen.selectionInfo.selectedNodeArray->len-1; i>=0; i--) {
-		gerbv_selection_item_t sItem = g_array_index (screen.selectionInfo.selectedNodeArray,
-				gerbv_selection_item_t, i);
-
-		gerbv_image_t *matchImage = (gerbv_image_t *) sItem.image;	
-		if (mainProject->file[index]->image == matchImage) {
-			g_array_remove_index (screen.selectionInfo.selectedNodeArray, index);
-		}
+		if (image == (gerbv_image_t *) sItem.image)
+			selection_clear_item_by_index (sel_info, i);
+		else
+			i++;
 	}
-	callbacks_update_selected_object_message (FALSE);
 }
 
 /* ------------------------------------------------------ */
@@ -536,57 +507,65 @@ render_create_cairo_buffer_surface () {
 }
 
 /* ------------------------------------------------------ */
-void
-render_find_selected_objects_and_refresh_display (gint activeFileIndex, gboolean eraseOldSelection){
+static void
+render_find_selected_objects_and_refresh_display (gint activeFileIndex,
+		enum selection_action action)
+{
+	enum draw_mode mode = FIND_SELECTIONS;
+
 	/* clear the old selection array if desired */
-	if ((eraseOldSelection)&&(screen.selectionInfo.selectedNodeArray->len))
-		g_array_remove_range (screen.selectionInfo.selectedNodeArray, 0,
-			screen.selectionInfo.selectedNodeArray->len);
+	if ((action == SELECTION_REPLACE)
+	 && (selection_length (&screen.selectionInfo) != 0))
+		selection_clear (&screen.selectionInfo);
+
+	if (action == SELECTION_TOGGLE)
+		mode = FIND_SELECTIONS_TOGGLE;
 
 	/* make sure we have a bufferSurface...if we start up in FAST mode, we may not
 	   have one yet, but we need it for selections */
-	if (!render_create_cairo_buffer_surface())
+	if (!render_create_cairo_buffer_surface ())
 		return;
 
 	/* call draw_image... passing the FILL_SELECTION mode to just search for
 	   nets which match the selection, and fill the selection buffer with them */
-	cairo_t *cr= cairo_create(screen.bufferSurface);	
-	gerbv_render_cairo_set_scale_and_translation(cr,&screenRenderInfo);
-	draw_image_to_cairo_target (cr, mainProject->file[activeFileIndex]->image,
-		1.0/MAX(screenRenderInfo.scaleFactorX, screenRenderInfo.scaleFactorY),
-		FIND_SELECTIONS, &screen.selectionInfo, &screenRenderInfo, TRUE,
-		mainProject->file[activeFileIndex]->transform, TRUE);
+	cairo_t *cr = cairo_create (screen.bufferSurface);	
+	gerbv_render_cairo_set_scale_and_translation (cr, &screenRenderInfo);
+	draw_image_to_cairo_target (cr,
+			mainProject->file[activeFileIndex]->image,
+			1.0/MAX (screenRenderInfo.scaleFactorX,
+				screenRenderInfo.scaleFactorY),
+			mode, &screen.selectionInfo, &screenRenderInfo, TRUE,
+			mainProject->file[activeFileIndex]->transform, TRUE);
 	cairo_destroy (cr);
-	/* if the selection array is empty, switch the "mode" to empty to make it
-	   easier to check if it is holding anything */
-	if (!screen.selectionInfo.selectedNodeArray->len)
-		screen.selectionInfo.type = GERBV_SELECTION_EMPTY;
+
 	/* re-render the selection buffer layer */
-	if (screenRenderInfo.renderType <= GERBV_RENDER_TYPE_GDK_XOR){
+	if (screenRenderInfo.renderType <= GERBV_RENDER_TYPE_GDK_XOR) {
 		render_refresh_rendered_image_on_screen ();
-	}
-	else {
-		render_selection_layer();
+	} else {
 		render_recreate_composite_surface ();
-		callbacks_force_expose_event_for_screen();
+		callbacks_force_expose_event_for_screen ();
 	}
 }
 
 /* ------------------------------------------------------ */
 void
-render_fill_selection_buffer_from_mouse_click (gint mouseX, gint mouseY, gint activeFileIndex,
-		gboolean eraseOldSelection) {
+render_fill_selection_buffer_from_mouse_click (gint mouseX, gint mouseY,
+		gint activeFileIndex, enum selection_action action)
+{
 	screen.selectionInfo.lowerLeftX = mouseX;
 	screen.selectionInfo.lowerLeftY = mouseY;
 	/* no need to populate the upperright coordinates for a point_click */
 	screen.selectionInfo.type = GERBV_SELECTION_POINT_CLICK;
-	render_find_selected_objects_and_refresh_display (activeFileIndex, eraseOldSelection);
+	render_find_selected_objects_and_refresh_display (
+			activeFileIndex, action);
 }
 
 /* ------------------------------------------------------ */
 void
 render_fill_selection_buffer_from_mouse_drag (gint corner1X, gint corner1Y,
-	gint corner2X, gint corner2Y, gint activeFileIndex, gboolean eraseOldSelection) {
+		gint corner2X, gint corner2Y,
+		gint activeFileIndex, enum selection_action action)
+{
 	/* figure out the lower left corner of the box */
 	screen.selectionInfo.lowerLeftX = MIN(corner1X, corner2X);
 	screen.selectionInfo.lowerLeftY = MIN(corner1Y, corner2Y);
@@ -595,11 +574,13 @@ render_fill_selection_buffer_from_mouse_drag (gint corner1X, gint corner1Y,
 	screen.selectionInfo.upperRightY = MAX(corner1Y, corner2Y);
 	
 	screen.selectionInfo.type = GERBV_SELECTION_DRAG_BOX;
-	render_find_selected_objects_and_refresh_display (activeFileIndex, eraseOldSelection);
+	render_find_selected_objects_and_refresh_display (
+			activeFileIndex, action);
 }
 
 /* ------------------------------------------------------ */
-void render_recreate_composite_surface () {
+void render_recreate_composite_surface ()
+{
 	gint i;
 	
 	if (!render_create_cairo_buffer_surface())
@@ -625,8 +606,10 @@ void render_recreate_composite_surface () {
 			}
 		}
 	}
+
 	/* render the selection layer at the end */
-	if (screen.selectionInfo.type != GERBV_SELECTION_EMPTY) {
+	if (selection_length (&screen.selectionInfo) != 0) {
+		render_selection ();
 		cairo_set_source_surface (cr, (cairo_surface_t *) screen.selectionRenderData,
 			                              0, 0);
 		cairo_paint_with_alpha (cr,1.0);
