@@ -1,5 +1,10 @@
 /* SCHEME.H */
 
+/** \file scheme.h
+    \brief Header info for the TinyScheme compiler
+    \ingroup gerbv
+*/
+
 #ifndef _SCHEME_H
 #define _SCHEME_H
 
